<?php
namespace App\Controller;

use App\Templates;
use App\Exception;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

use SimpleXMLElement;

class WebService {

    private $util;
    private $integrationService;
    
    private $basePath;
    private $ordersPath;        
    private $processedPath;     
    private $errPath;
    
    private $mailList;
    private $serverName;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util,
                                 \App\Service\IntegrationService $integrationService) {

        $this->connection         = $connection;
        $this->util               = $util;
        $this->integrationService = $integrationService;
    }

    public function sendOrder(Request $request, Response $response) {
        try {
        
            $req          = $request->getParsedBody();
            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            if (empty($req)) {
                throw new Exception\ParamsException("Nenhum parametro foi enviado na requisição.");
            }
            
            $xml = $req->asXML();
            
            $fileName = date('d-m-Y H:i:s') . " - " . str_pad($req->empresa->data->Id, 4, 0, STR_PAD_LEFT) . "-" . $req->Id;

            $this->util->writeLog('events', $fileName, $xml);
            $xmlString = simplexml_load_string($xml);
            
            if ($xmlString === false) {
                throw new Exception\ParamsException("Parametros enviado não é um XML válido.");
            }
            
            $array = json_decode(json_encode($xmlString), true);
            
            $requestBody = [
                "id"                  => $array['Id'],
                "name"                => $array['Name'],
                "telefone"            => $array['telefone'],
                "rua"                 => $array['Rua'],
                "numero"              => $array['Numero'],
                "bairro"              => $array['Bairro'],
                "cep"                 => isset($array['CEP']) ? $array['CEP'] : '',
                "municipio"           => isset($array['Municipio']) ? $array['Municipio'] : '',
                "estado"              => isset($array['Estado']) ? $array['Estado'] : '',
                "origem"              => $array['Origem'],
                "dataHoraPedido"      => $array['DataHoraPedido'],
                "dataHoraPronto"      => $array['DataHoraPronto'],
                "DataHoraAgendamento" => $array['DataHoraAgendamento'],
                "valor"               => bcadd(str_replace(',', '.', $array['Valor']), '0', 2),
                "status"              => $array['status'],
                "numeroVenda"         => $array['numeroVenda'],
                "itensPedido"         => [],
                "pagamentos"          => []
            ];
            
            $requestBody['entregador'] = [
                'data' => [
                    'id'   => $array['entregador']['data']['Id'],
                    'name' => $array['entregador']['data']['Name'],
                    'cpf'  => $array['entregador']['data']['cpf'],
                ]
            ];
                
            $requestBody['rota'] = [
                'data' => [
                    'id'             => $array['Rota']['data']['Id'],
                    'dataHoraInicio' => $array['Rota']['data']['DataHoraInicio'],
                    'usuario'        => $array['Rota']['data']['Usuario'],
                ]
            ];
            
            $requestBody['empresa'] = [
                'data' => [
                    'id'      => $array['empresa']['data']['Id'],
                    'name'    => $array['empresa']['data']['Name'],
                    'mall_Id' => $array['empresa']['data']['Mall_id'],
                ]
            ];
            
            $requestBody['itensPedido']['data'] = [];
            
            if(isset($array['ItensPedido']['data'][0])) {
                $prods = $array['ItensPedido'];
            } else {
                $prods['data'][0] = $array['ItensPedido']['data'];
            }
            
            foreach($prods['data'] as $item) {
                $arr = [
                    'id'         => $item['Id'],
                    'codigo'     => $item['codigo'],
                    'descricao'  => $item['Descricao'],
                    'valorUnit'  => bcadd(str_replace(',', '.', $item['ValorUnit']), '0', 2),
                    'quantidade' => str_replace(',', '.', $item['Quantidade']),
                    'valorTotal' => bcadd(str_replace(',', '.', $item['ValorTotal']), '0', 2),
                    'observacao' => empty($item['Observacao']) ? '' : $item['Observacao']
                ];
                
                array_push($requestBody['itensPedido']['data'], $arr);
            }
            
            $requestBody['pagamentos']['data'] = [];
            
            if(isset($array['Pagamentos']['data'][0])) {
                $payments = $array['Pagamentos'];
            } else {
                $payments['data'][0] = $array['Pagamentos']['data'];
            }
            
            if(isset($arr))
                unset($arr);
            
            foreach($payments['data'] as $payment) {
                $arr = [
                    'id'        => $payment['Id'],
                    'sequencia' => $payment['Sequencia'],
                    'codigo'    => $payment['codigo'],
                    'descricao' => $payment['Descricao'],
                    'valor'     => bcadd(str_replace(',', '.', $payment['Valor']), '0', 2),
                    'troco'     => bcadd(str_replace(',', '.', $payment['troco']), '0', 2),
                    'tipo'      => $this->getReceiptType($payment['tipo'])
                ];
                
                array_push($requestBody['pagamentos']['data'], $arr);
            }
            
            $requestHeader = [
                "Content-Type: application/json"
            ];
            
            $integrationType = $config['integrator'];
            
            switch($integrationType) {
                case 'LET_EXPRESS':
                    $requestResponse = $this->sendRequest('/letExpress', 'POST', $requestHeader, $requestBody);
                    break;
                    
                case 'ROTEIRIZADOR':
                    $requestResponse = $this->sendRequest('/berp/Pedido', 'POST', $requestHeader, $requestBody);
                    break;
                    
                case 'BOX_DELIVERY':
                    $requestHeader = [
                        'Content-Type: application/json',
                        'Authorization: ef5f99d1d500a984f58803ffcb7c3007'
                    ];
                    
                    $requestResponse = $this->sendRequest('/teknisa', 'POST', $requestHeader, $requestBody);
                    
                    break;
                    
                default:
                    throw new \Exception('Tipo de integrador logistico não mapeado.');
                
            }

            if($requestResponse['success'] === false) {
                $message = "Erro ao enviar o pedido " . $requestBody['id'] . " para a API integrador logistico.";
                
                $responseXml = $this->integrationService->buildXml($requestBody, $message, false);
                
                $fileName = $requestBody['id'].".json";
                $this->util->writeLog('error', $fileName, $requestBody);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao enviar o pedido " . $requestBody['id'] . " para a API do integrador logistico." . "\n");
            } else {
                $message = "Pedido " . $requestBody['id'] . " enviado para o integrador logistico com sucesso.";
                
                $responseXml = $this->integrationService->buildXml($requestBody, $message, true, true);
                
                $fileName = $requestBody['id'].".json";
                $this->util->writeLog('sale', $fileName, $requestBody);
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido " . $requestBody['id'] . " enviado para a API do integrador logistico com sucesso." . "\n");
            }
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXml([], "Ocorreu um erro inesperado: " . $e->getMessage(), false);
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function getOrder(Request $request, Response $response) {
        try {
        
            $req = $request->getParsedBody();

            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            if(empty($array)){
                throw new \Exception('Parametros de entrada invalidos.');
            }
            
            $requestHeader = [
                "Content-Type: application/json"
            ];
            
            $requestResponse = $this->sendRequest('/berp/Pedido(' . $array["Id"] . ')', 'GET', $requestHeader, null);
            
            if($requestResponse['success'] === false){
                $message = "Erro ao buscar o pedido " . $array["Id"] . " na API Roteirizador.";
                
                $responseXml = $this->integrationService->buildOrderXml($requestResponse['curlResponse'], $message, false);
                
                $fileName = $array["Id"].".txt";
                $this->util->writeLog('error', $fileName, "");
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao buscar o pedido " . $array["Id"] . " na API Roteirizador." . "\n");
            } else{
                $message = "Pedido " . $array["Id"] . " buscado na API Roteirizador com sucesso.";
                
                $responseXml = $this->integrationService->buildOrderXml($requestResponse['curlResponse'], $message, true);
                
                $fileName = $array["Id"].".txt";
                $this->util->writeLog('sale', $fileName, "");
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido " . $array["Id"] . " buscado na API Roteirizador com sucesso." . "\n");
            }
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXml([], "Ocorreu um erro inesperado: " . $e->getMessage(), false);
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function patchOrder(Request $request, Response $response) {
        try {
        
            $req = $request->getParsedBody();

            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            if(empty($array)){
                throw new \Exception('Parametros de entrada invalidos.');
            }
            
            $requestBody = [
                "Id"             => $array["Id"],
                "Name"           => $array["Name"],
                "telefone"       => $array["telefone"],
                "Rua"            => $array["Rua"],
                "Numero"         => $array["Numero"],
                "Bairro"         => $array["Bairro"],
                "Origem"         => $array["Origem"],
                "DataHoraPedido" => $array["DataHoraPedido"],
                "DataHoraPronto" => $array["DataHoraPronto"],
                "Valor"          => bcadd($array["Valor"], '0', 2),
                "status"         => $array["status"],
                "numeroVenda"    => $array["numeroVenda"],
                "entregador"     => $array["entregador"],
                "Rota"           => $array["Rota"],
                "empresa"        => $array["empresa"],
                "ItensPedido"    => $array["ItensPedido"],
                "Pagamentos"     => $array["Pagamentos"]
            ];
            
            foreach($requestBody['ItensPedido'] as &$orderItem){
                if(empty($orderItem['Observacao'])){
                    $orderItem['Observacao'] = '';
                }
            }
            
            $requestHeader = [
                "Content-Type: application/json"
            ];
            
            $requestResponse = $this->sendRequest('/berp/Pedido(' . $array["Id"] . ')', 'PATCH', $requestHeader, $requestBody);
            
            if($requestResponse['success'] === false){
                $message = "Erro ao atualizar o pedido " . $requestBody['Id'] . " na API Roteirizador.";
                
                $responseXml = $this->integrationService->buildXml($requestResponse['curlResponse'], $message, false);
                
                $fileName = $requestBody['Id'].".txt";
                $this->util->writeLog('error', $fileName, json_encode($requestBody));
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao atualizar o pedido " . $requestBody['Id'] . " na API Roteirizador." . "\n");
            } else{
                $message = "Pedido " . $requestBody['Id'] . " atualizado na API Roteirizador com sucesso.";
                
                $responseXml = $this->integrationService->buildXml($requestResponse['curlResponse'], $message, true);
                
                $fileName = $requestBody['Id'].".txt";
                $this->util->writeLog('sale', $fileName, json_encode($requestBody));
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido " . $requestBody['Id'] . " atualizado na API Roteirizador com sucesso." . "\n");
            }
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXml([], "Ocorreu um erro inesperado: " . $e->getMessage(), false);
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    public function isReadyToDeliver(Request $request, Response $response) {
        try {
        
            $req = $request->getParsedBody();

            $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
            $config       = json_decode($configsFile, true);
            
            $json = json_encode($req);
            $array = json_decode($json, TRUE);
            
            if(empty($array)){
                throw new \Exception('Parametros de entrada invalidos.');
            }
            
            $requestBody = [
                "identificador" => $array["identificador"],
                "empresa"       => $array["empresa"]
            ];
            
            $requestHeader = [
                "Content-Type: application/json"
            ];
            
            $requestResponse = $this->sendRequest('/berp/MyServices/isReadyOrder', 'POST', $requestHeader, $requestBody);
            
            if($requestResponse['success'] === false){
                $message = "Erro ao despachar o pedido " . $requestBody['identificador'] . " na API Roteirizador.";
                
                $responseXml = $this->integrationService->buildXml($requestResponse['curlResponse'], $message, false);
                
                $fileName = $requestBody['identificador'].".txt";
                $this->util->writeLog('error', $fileName, json_encode($requestBody));
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Erro ao despachar o pedido " . $requestBody['identificador'] . " na API Roteirizador." . "\n");
            } else{
                $message = "Pedido " . $requestBody['identificador'] . " marcado como pronto para retirada na API Roteirizador com sucesso.";
                
                $responseXml = $this->integrationService->buildXml($requestResponse['curlResponse'], $message, true);
                
                $fileName = $requestBody['identificador'].".txt";
                $this->util->writeLog('sale', $fileName, json_encode($requestBody));
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Pedido " . $requestBody['identificador'] . " marcado como pronto para retirada na API Roteirizador com sucesso." . "\n");
            }
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
            
        } catch (\Exception $e) {
            $responseXml = $this->integrationService->buildXml([], "Ocorreu um erro inesperado: " . $e->getMessage(), false);
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Ocorreu um erro inesperado: " . $e->getMessage() . "\n");
            
            return $response->withHeader('Content-Type', "application/xml")->write($responseXml);
        }
    }
    
    private function getCurrentAuthToken(){
        $tokenFile = file_get_contents(__DIR__ . "/../../service/authToken.json");
        $tokenData = json_decode($tokenFile, true);
        
        $isTokenValid = false;
        
        $currentTime = strtotime(date('d-m-Y H:i:s'));
        
        if(!empty($tokenData)){
            if($currentTime - $tokenData['timestamp'] < 3000){
                $isTokenValid = true;
            }
        }
        
        if($isTokenValid){
            
            return $tokenData['token'];
            
        } else{
            throw new \Exception('Current auth token is not valid.');
        }
    }
    
    public function renewAuthToken(){
        $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." Iniciando servico de renovacao de token." . PHP_EOL);
        
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $authUrl = !empty($config['authUrl']) ? $config['authUrl'] : null;
        $apiKey = !empty($config['apiKey']) ? $config['apiKey'] : null;
        $authEmail = !empty($config['authEmail']) ? $config['authEmail'] : null;
        $authPass = !empty($config['authPass']) ? $config['authPass'] : null;
        
        //valid token check
        $tokenFile = file_get_contents(__DIR__ . "/../../service/authToken.json");
        $tokenData = json_decode($tokenFile, true);
        
        $isTokenValid = false;
        
        $currentTime = strtotime(date('d-m-Y H:i:s'));
        
        if(!empty($tokenData)){
            if($currentTime - $tokenData['timestamp'] < 3000){
                $isTokenValid = true;
                $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." O token atual ainda e valido." . PHP_EOL);
            }
        }
        
        if(!$isTokenValid){
            $requestUrl = $authUrl."?key=".$apiKey;
            
            $requestHeader = [
                "Content-Type: application/json"
            ];
            
            $requestBody = [
                "email" => $authEmail,
                "password" => $authPass,
                "returnSecureToken" => true
            ];
            
            $curl = curl_init();
            
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => "POST",
                CURLOPT_POSTFIELDS      => json_encode($requestBody),
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
            
            $curlResponse = curl_exec($curl);
            $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
    
            if (!empty(curl_error($curl)) || ($httpCode != 200) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
                $fileName = "EXP-".date("H-i-s") . ".txt";
    
                $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                    'eventTime' => date("d/m/Y H:i:s"),
                    'header'    => $requestHeader,
                    'url'       => $requestUrl,
                    'method'    => "POST",
                    'params'    => json_encode($requestBody),
                    'httpCode'  => $httpCode,
                    'error'     => curl_error($curl),
                    'response'  => $curlResponse
                ]));
            }
            
            curl_close($curl);
            
            $return = json_decode($curlResponse, true);
            
            $newToken = [
                "token" => $return['idToken'],
                "timestamp" => strtotime(date('d-m-Y H:i:s'))
            ];
            
            $filePath = __DIR__ . "/../../service/authToken.json";
            file_put_contents($filePath, json_encode($newToken));
            
            $this->util->writeLog("execution", "", date("d/m/Y H:i:s")." O token foi renovado com sucesso." . PHP_EOL);
        }
    }
    
    private function sendRequest($route, $method, $requestHeader, $requestBody, $query = null){
        $configsFile  = file_get_contents(__DIR__ . "/../../configs/configs.json");
        $config       = json_decode($configsFile, true);
        
        $url = !empty($config['url']) ? $config['url'] : null;
        
        $requestUrl = $url.$route;
        
        if(!empty($query)){
            $requestUrl = $requestUrl.'?'.$query;
        }
        
        $curl = curl_init();
        
        if(empty($requestBody)){
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        } else{
            curl_setopt_array($curl, [
                CURLOPT_URL             => $requestUrl,
                CURLOPT_RETURNTRANSFER  => true,
                CURLOPT_ENCODING        => '',
                CURLOPT_CONNECTTIMEOUT  => 5,
                CURLOPT_TIMEOUT         => 90,
                CURLOPT_FOLLOWLOCATION  => true,
                CURLOPT_SSL_VERIFYPEER  => false,
                CURLOPT_USERAGENT       => "Mozilla/5.0 (Windows NT 6.2; WOW64; rv:17.0) Gecko/20100101 Firefox/17.0",
                CURLOPT_HTTP_VERSION    => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST   => $method,
                CURLOPT_POSTFIELDS      => json_encode($requestBody),
                CURLOPT_HTTPHEADER      => $requestHeader
            ]);
        }
        
        $curlResponse = curl_exec($curl);
        $httpCode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        $success = true;

        if (!empty(curl_error($curl)) || (($httpCode != 200) && ($httpCode != 201)) || (stripos($curlResponse, "<html>")) || (stripos($curlResponse, "Fatal error"))) {
            $fileName = "EXP-".date("H-i-s") . ".txt";

            $this->util->writeLog('network', $fileName, $this->util->formatNetworkError([
                'eventTime' => date("d/m/Y H:i:s"),
                'header'    => $requestHeader,
                'url'       => $url.$route,
                'method'    => $method,
                'params'    => json_encode($requestBody),
                'httpCode'  => $httpCode,
                'error'     => curl_error($curl),
                'response'  => $curlResponse
            ]));
            
            $success = false;
        }
        
        curl_close($curl);
        
        $resp = [
            'success' => $success,
            'curlResponse' => json_decode($curlResponse, true)
        ];
        
        return $resp;
    }

    private function writeEventsLog($events) {
        foreach($events as $event) {

            $reference = $event['correlationId'];
            $eventCode = $event['code'];

            $fileName = round(microtime(true)*1000) . "-$eventCode.json";
            $this->util->writeLog("events", $fileName, $event);
        }
    }
    
    public function welcome(Request $request, Response $response) {
        $sql        = "SELECT CDFILIAL FROM FILIAL";
        $result     = $this->connection->fetchAll($sql);

        $versionInfo = $this->getVersionInfo();

        return $response->withJson([
            'version'     => $versionInfo['version'],
            'revision'    => $versionInfo['revision'],
            'date'        => $versionInfo['date'],
            'branchs'     => count($result)
        ]);
    }

    private function getVersionInfo() {
        $file = file_get_contents(__DIR__ . "/../../version.json");
        return json_decode($file, true);
    }

    private function getClientName() {
        global $CLIENT_NAME;
        return $CLIENT_NAME;
    }
    
    private function getReceiptType($idtiporece) {
        switch($idtiporece) {
            case  '1':
                $return = 'CREDITO';
                break;
                
            case '2':
                $return = 'DEBITO';
                break;
            
            case '4':
                $return = 'DINHEIRO';
                break;
            
            case '5':
                $return = 'TICKET';
                break;
                
            case 'G':
                $return = 'TICKET';
                break;
            
            case 'F':
                $return = 'TICKET';
                break;
            
            case 'C':
                $return = 'ONLINE';
                break;
                    
            default:
                $return = 'PDV';
        }
        
        return $return;
    }
}
